indexing

	description:

		"Black box tests for interpreter"

	copyright: "Copyright (c) 2005, Andreas Leitner and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

deferred class ERL_G_TEST_INTERPRETER

inherit

	TS_TEST_CASE

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

feature -- Test

	test_1 is
			-- Run 'interpreter_example'.
		do
			assert_interpreter ("test run 1", input_1_filename, output_1_filename)
			assert_interpreter ("test run 2", input_2_filename, output_2_filename)
			assert_interpreter ("test run 3", input_3_filename, output_3_filename)
		end

feature -- Access

	program_name: STRING is "interpreter_example"
			-- Program name

feature {NONE} -- Filenames

	program_dirname: STRING is
			-- Name of program source directory
		once
			Result := file_system.nested_pathname ("${ERL_G}", <<"test", "runtime", program_name>>)
			Result := Execution_environment.interpreted_string (Result)
		end

	program_exe: STRING is
			-- Name of program executable filename
		do
			Result := file_system.pathname (program_dirname, program_name + file_system.exe_extension)
		ensure
			program_exe_not_void: Result /= Void
			program_exe_not_empty: Result.count > 0
		end
		
	melt_dirname: STRING is
			-- Name of directory where melt file is stored
		do
			Result := program_dirname
		ensure
			melt_dir_not_void: Result /= Void
			melt_dir_not_empty: Result.count > 0
		end

	data_dirname: STRING is
			-- Name of directory containing expected output files
		once
			Result := file_system.pathname (program_dirname, "data")
			Result := Execution_environment.interpreted_string (Result)
		ensure
			data_dirname_not_void: Result /= Void
			data_dirname_not_empty: Result.count > 0
		end

	input_1_filename: STRING is
			-- Name of first test input file
		once
			Result := file_system.pathname (data_dirname, "test_input_1")
		ensure
			testoutput1_filename_not_void: Result /= Void
			testoutput1_filename_not_empty: Result.count > 0
		end

	output_1_filename: STRING is
			-- Name of first test expected output file
		once
			Result := file_system.pathname (data_dirname, "test_output_1")
		ensure
			testoutput1_filename_not_void: Result /= Void
			testoutput1_filename_not_empty: Result.count > 0
		end

	input_2_filename: STRING is
			-- Name of first test input file
		once
			Result := file_system.pathname (data_dirname, "test_input_2")
		ensure
			testoutput1_filename_not_void: Result /= Void
			testoutput1_filename_not_empty: Result.count > 0
		end

	output_2_filename: STRING is
			-- Name of first test expected output file
		once
			Result := file_system.pathname (data_dirname, "test_output_2")
		ensure
			testoutput1_filename_not_void: Result /= Void
			testoutput1_filename_not_empty: Result.count > 0
		end

	input_3_filename: STRING is
			-- Name of first test input file
		once
			Result := file_system.pathname (data_dirname, "test_input_3")
		ensure
			testoutput1_filename_not_void: Result /= Void
			testoutput1_filename_not_empty: Result.count > 0
		end

	output_3_filename: STRING is
			-- Name of first test expected output file
		once
			Result := file_system.pathname (data_dirname, "test_output_3")
		ensure
			testoutput1_filename_not_void: Result /= Void
			testoutput1_filename_not_empty: Result.count > 0
		end

feature {NONE} -- Assertions routines

	assert_interpreter (a_tag: STRING; an_input_filename: STRING; an_output_filename: STRING) is
			-- Assert `a_condition'.
		require
			a_tag_not_void: a_tag /= Void
			an_input_filename_not_void: an_input_filename /= Void
			an_output_filename_not_void: an_output_filename /= Void
		do
			execution_environment.set_variable_value ("MELT_PATH", melt_dirname)
			assert_execute (program_exe + " > interpreter_output.txt < " + an_input_filename)
			assert_files_equal (a_tag, "interpreter_output.txt", an_output_filename)
		end

end